@@ -1,8 +1,9 @@
 from sklearn import ensemble, neural_network, linear_model, svm, neighbors
 from .base import *
+import numpy as np
 
 from warnings import simplefilter, filterwarnings
-from sklearn.exceptions import ConvergenceWarning
+from sklearn.exceptions import ConvergenceWarning, DataConversionWarning
 simplefilter("ignore", category=ConvergenceWarning)
 filterwarnings("ignore", category=ConvergenceWarning, message="^Maximum number of iteration reached")
 filterwarnings("ignore", category=ConvergenceWarning, message="^Liblinear failed to converge")
